@@ -3,7 +3,7 @@ Package for data processing in Python: https://github.com/WinVector/data_algebra
 """
 
 __docformat__ = "restructuredtext"
-__version__ = "1.4.3"
+__version__ = "1.4.5"
 
 __doc__ = """
 `data_algebra`<https://github.com/WinVector/data_algebra> is a piped data wrangling system
@@ -16,8 +16,5 @@ the same transforms for Pandas DataFrames.
 the `rquery`<https://github.com/WinVector/rquery> and `rqdatatable`<https://github.com/WinVector/rqdatatable> packages.
 """
 
-import data_algebra.pandas_model
-
-
-# set up what pandas supplier we are using
-default_data_model = data_algebra.pandas_model.PandasModel()
+# import for easy access for package users
+from data_algebra.data_ops import TableDescription, SQLNode, describe_table, descr, data, ex
