@@ -7,8 +7,7 @@ import re
 from typing import Iterable, List, Optional
 
 import numpy
-
-import data_algebra
+import data_algebra.pandas_model
 import data_algebra.util
 
 
@@ -36,7 +35,7 @@ class RecordSpecification:
         :param local_data_model: data.frame data model
         """
         if local_data_model is None:
-            local_data_model = data_algebra.default_data_model
+            local_data_model = data_algebra.pandas_model.default_data_model
         control_table = control_table.reset_index(inplace=False, drop=True)
         if control_table.shape[0] < 1:
             raise ValueError("control table should have at least 1 row")
@@ -194,7 +193,7 @@ def blocks_to_rowrecs(data, *, blocks_in: RecordSpecification, local_data_model=
     if len(ck) != len(set(ck)):
         raise ValueError("blocks_in can not have duplicate content keys")
     if local_data_model is None:
-        local_data_model = data_algebra.default_data_model
+        local_data_model = data_algebra.pandas_model.default_data_model
     data = data.reset_index(drop=True)
     missing_cols = set(blocks_in.control_table_keys).union(blocks_in.record_keys) - set(
         data.columns
@@ -281,7 +280,7 @@ def rowrecs_to_blocks(
     """
     assert isinstance(blocks_out, RecordSpecification)
     if local_data_model is None:
-        local_data_model = data_algebra.default_data_model
+        local_data_model = data_algebra.pandas_model.default_data_model
     data = data.reset_index(drop=True)
     missing_cols = set(blocks_out.record_keys) - set(data.columns)
     if len(missing_cols) > 0:
@@ -334,7 +333,10 @@ def rowrecs_to_blocks(
                     nvals = numpy.asarray(dtemp[dcol])
                     if len(nvals) < 1:
                         nvals = [None] * numpy.sum(want)
-                    res.loc[want, vk] = nvals
+                    if numpy.all(want):
+                        res[vk] = nvals  # get around Pandas future warning
+                    else:
+                        res.loc[want, vk] = nvals
     # see about promoting composite columns to numeric
     for vk in set(value_keys):
         converted = local_data_model.to_numeric(res[vk], errors="coerce")
@@ -344,7 +346,7 @@ def rowrecs_to_blocks(
             res[vk] = converted
     if data.shape[0] < 1:
         # empty input produces emtpy output (with different column structure)
-        res = res.iloc[range(0), :].reset_index(drop=True)
+        res = res.iloc[range(0), :].reset_index(inplace=False, drop=True)
     if data.shape[0] <= 0:
         res = res.loc[range(0), :]
         res = res.reset_index(inplace=False, drop=True)
@@ -431,7 +433,7 @@ class RecordMap:
         :return: example result data frame.
         """
         if local_data_model is None:
-            local_data_model = data_algebra.default_data_model
+            local_data_model = data_algebra.pandas_model.default_data_model
         if self.blocks_in is not None:
             example = self.blocks_in.control_table.copy()
             nrow = example.shape[0]
@@ -461,7 +463,7 @@ class RecordMap:
         if len(unknown) > 0:
             raise ValueError("missing required columns: " + str(unknown))
         if local_data_model is None:
-            local_data_model = data_algebra.default_data_model
+            local_data_model = data_algebra.pandas_model.default_data_model
         X = X.reset_index(drop=True)
         if self.blocks_in is not None:
             X = blocks_to_rowrecs(
@@ -651,7 +653,7 @@ def pivot_blocks_to_rowrecs(
     """
 
     if local_data_model is None:
-        local_data_model = data_algebra.default_data_model
+        local_data_model = data_algebra.pandas_model.default_data_model
     control_table = local_data_model.data_frame(
         {
             attribute_key_column: record_value_columns,
@@ -687,7 +689,7 @@ def pivot_rowrecs_to_blocks(
     """
 
     if local_data_model is None:
-        local_data_model = data_algebra.default_data_model
+        local_data_model = data_algebra.pandas_model.default_data_model
     control_table = local_data_model.data_frame(
         {
             attribute_key_column: record_value_columns,
@@ -730,7 +732,7 @@ def pivot_specification(
     assert len(known_cols) == len(set(known_cols))
     record_map = RecordMap(
         blocks_in=RecordSpecification(
-            control_table=data_algebra.pandas_model.pd.DataFrame(
+            control_table=data_algebra.pandas_model.default_data_model.pd.DataFrame(
                 {
                     col_name_key: value_cols,
                     col_value_key: value_cols,
@@ -770,7 +772,7 @@ def unpivot_specification(
     assert len(known_cols) == len(set(known_cols))
     record_map = RecordMap(
         blocks_out=RecordSpecification(
-            control_table=data_algebra.pandas_model.pd.DataFrame(
+            control_table=data_algebra.pandas_model.default_data_model.pd.DataFrame(
                 {
                     col_name_key: value_cols,
                     col_value_key: value_cols,
