@@ -11,8 +11,7 @@ from typing import Dict, Set, Tuple
 
 import pandas.io.sql
 
-import data_algebra
-
+import data_algebra.pandas_model
 import data_algebra.near_sql
 import data_algebra.expr_rep
 import data_algebra.util
@@ -797,7 +796,7 @@ class DBModel:
         union_all_term_end: str = ")",
     ):
         if local_data_model is None:
-            local_data_model = data_algebra.default_data_model
+            local_data_model = data_algebra.pandas_model.default_data_model
         self.local_data_model = local_data_model
         if sql_formatters is None:
             sql_formatters = {}
@@ -845,7 +844,7 @@ class DBModel:
 
     def prepare_connection(self, conn):
         """
-        Do any augmentation or preperation of a database connection. Example: adding stored procedures.
+        Do any augmentation or preparation of a database connection. Example: adding stored procedures.
         """
         pass
 
@@ -855,7 +854,7 @@ class DBModel:
     def execute(self, conn, q):
         """
 
-        :param conn: database connectionex
+        :param conn: database connection
         :param q: sql query
         """
         if isinstance(q, data_algebra.data_ops.ViewRepresentation):
