@@ -3,7 +3,7 @@ Base class for adapters for Pandas-like APIs
 """
 
 from abc import ABC
-from typing import Any, Callable, Dict, Optional
+from typing import Any, Callable, Dict, List, Optional
 import datetime
 import types
 import numbers
@@ -21,6 +21,47 @@ import data_algebra.connected_components
 # TODO: possibly import dask, Nvidia Rapids, modin, datatable versions
 
 
+def none_mark_scalar_or_length(v) -> Optional[int]:
+    """
+    Test if item is a scalar (returning None) if it is, else length of object.
+
+    :param v: value to test
+    :return: None if value is a scalar, else length.
+    """
+    # get some of the obvious types, and str (as str doesn't throw on len)
+    if isinstance(v, (type(None), str, int, float)):
+        return None  # obvious scalar
+    # len() throws on scalars other than str
+    try:
+        return len(v)
+    except TypeError:
+        return None  # len() failed, probably a scalar
+
+        
+def promote_scalar_to_array(vi, *, target_len: int) -> List:
+    """
+    Convert a scalar into a vector. Pass a non-trivial array through.
+
+    :param vi: value to promote to scalar
+    :target_len: length for vector
+    :return: list
+    """
+    assert isinstance(target_len, int)
+    assert target_len >= 0
+    if target_len <= 0:
+        return []
+    len_v = none_mark_scalar_or_length(vi)
+    # noinspection PyBroadException
+    if len_v is None:
+        return [vi] * target_len  # scalar
+    if len_v == target_len:
+        return vi
+    if len_v == 1:
+        return [vi[0]] * target_len  # TODO: see if we can eliminate this one
+    else:
+        raise ValueError("incompatible column lengths")
+
+
 def _negate_or_subtract(*args):
     if len(args) == 1:
         return numpy.negative(args[0])
@@ -29,52 +70,6 @@ def _negate_or_subtract(*args):
     raise ValueError("unexpected number of arguments in _negate_or_subtract")
 
 
-def _calc_date_diff(x0, x1):
-    # x is a pandas Series or list of datetime.date compatible types
-    x0 = data_algebra.default_data_model.pd.Series(x0)
-    x1 = data_algebra.default_data_model.pd.Series(x1)
-    x0_dates = data_algebra.default_data_model.pd.to_datetime(x0).dt.date.copy()
-    x1_dates = data_algebra.default_data_model.pd.to_datetime(x1).dt.date.copy()
-    deltas = [(x0_dates[i] - x1_dates[i]).days for i in range(len(x0_dates))]
-    return deltas
-
-
-def _calc_base_Sunday(x):
-    # x is a pandas Series or list of datetime.date compatible types
-    x = data_algebra.default_data_model.pd.Series(x)
-    x_dates = data_algebra.default_data_model.pd.to_datetime(x).dt.date.copy()
-    res = [xi - datetime.timedelta(days=(xi.weekday() + 1) % 7) for xi in x_dates]
-    return res
-
-
-def _calc_week_of_Year(x):
-    # x is a pandas Series or list of datetime.date compatible types
-    # TODO: better impl
-    # Note was getting inconsistent results on vectorized methods
-    x = data_algebra.default_data_model.pd.Series(x)
-    x_dates = data_algebra.default_data_model.pd.to_datetime(x).dt.date.copy()
-    cur_dates = [datetime.date(dti.year, dti.month, dti.day) for dti in x_dates]
-    base_dates = [datetime.date(dti.year, 1, 1) for dti in x_dates]
-    base_dates = _calc_base_Sunday(base_dates)
-    deltas = [(cur_dates[i] - base_dates[i]).days for i in range(len(cur_dates))]
-    res = [di // 7 for di in deltas]
-    res = numpy.maximum(res, 1)
-    return res
-
-
-def _coalesce(a, b):
-    a_is_series = isinstance(a, data_algebra.default_data_model.pd.Series)
-    b_is_series = isinstance(b, data_algebra.default_data_model.pd.Series)
-    if (not a_is_series) and (not b_is_series):
-        raise ValueError("at least one argument must be a Pandas series")
-    if not a_is_series:
-        a = data_algebra.default_data_model.pd.Series(numpy.array([a] * len(b)))
-    if not b_is_series:
-        b = data_algebra.default_data_model.pd.Series(numpy.array([b] * len(a)))
-    res = a.combine_first(b)
-    return res
-
-
 def _type_safe_equal(a, b):
     # Could try numpy.logical_and(numpy.greater_equal(a, b), numpy.less_equal(a, b)).
     # However, None >= None throws
@@ -108,35 +103,6 @@ def _type_safe_is_in(a, b):
     return numpy.isin(a, b)
 
 
-def _map_v(a, value_map, default_value=None):
-    """Map values to values."""
-    if len(value_map) > 0:
-        type_a = data_algebra.util.guess_carried_scalar_type(a)
-        type_k = {
-            data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.keys()
-        }
-        if not data_algebra.util.compatible_types(type_k):
-            raise TypeError(f"multiple types in dictionary keys: {type_k} in mapv()")
-        type_k = list(type_k)[0]
-        if not data_algebra.util.compatible_types([type_a, type_k]):
-            raise TypeError(f"can't map {type_a} from a dict of {type_k}'s in mapv()")
-        type_v = {
-            data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.values()
-        }
-        if not data_algebra.util.compatible_types(type_v):
-            raise TypeError(f"multiple types in dictionary values: {type_v} in mapv()")
-        type_v = list(type_v)[0]
-        if default_value is not None:
-            type_d = data_algebra.util.guess_carried_scalar_type(default_value)
-            if not data_algebra.util.compatible_types([type_d, type_v]):
-                raise TypeError(f"default is {type_d} for {type_v} values in mapv()'s")
-    # https://pandas.pydata.org/docs/reference/api/pandas.Series.map.html
-    a = data_algebra.default_data_model.pd.Series(a)
-    a = a.map(value_map, na_action="ignore")
-    a[data_algebra.default_data_model.bad_column_positions(a)] = default_value
-    return numpy.array(a.values)
-
-
 def _k_and(*args):
     res = args[0]
     for i in range(1, len(args)):
@@ -176,120 +142,6 @@ def _where_expr(*args):
     return numpy.where(cond, a, b)
 
 
-def _if_else_expr(*args):
-    """
-    if_else(cond, a, b) returns a for positions where cond is True, b when cond is False, None otherwise.
-    """
-    assert len(args) == 3
-    cond = args[0]
-    a = args[1]
-    b = args[2]
-    res = numpy.where(cond, a, b)
-    bad_posns = data_algebra.default_data_model.bad_column_positions(cond)
-    if numpy.any(bad_posns):
-        res[bad_posns] = None
-    return res
-
-
-def populate_impl_map(data_model) -> Dict[str, Callable]:
-    """
-    Map symbols to implementations.
-    """
-    impl_map = {
-        "==": _type_safe_equal,
-        "=": _type_safe_equal,
-        "!=": _type_safe_not_equal,
-        "<>": _type_safe_not_equal,
-        "<": numpy.less,  # already checks types
-        "<=": numpy.less_equal,  # already checks types
-        ">": numpy.greater,  # already checks types
-        ">=": numpy.greater_equal,  # already checks types
-        "+": _k_add,
-        "-": _negate_or_subtract,
-        "neg": numpy.negative,
-        "*": _k_mul,
-        "/": numpy.divide,
-        "//": numpy.floor_divide,
-        "%/%": numpy.divide,
-        "%": numpy.mod,
-        "**": numpy.power,
-        "and": _k_and,
-        "&": numpy.bitwise_and,
-        "or": _k_or,
-        "|": _k_or,
-        "xor": numpy.logical_xor,
-        "^": numpy.logical_xor,
-        "not": numpy.bitwise_or,
-        "where": _where_expr,
-        "if_else": _if_else_expr,
-        "is_nan": data_model.isnan,
-        "is_inf": data_model.isinf,
-        "is_null": data_model.isnull,
-        "is_bad": data_model.bad_column_positions,
-        "is_in": _type_safe_is_in,
-        "concat": lambda a, b: numpy.char.add(
-            numpy.asarray(a, dtype=str), numpy.asarray(b, dtype=str)
-        ),
-        "coalesce": lambda a, b: _coalesce(a, b),  # assuming Pandas series
-        "connected_components": lambda a, b: data_algebra.connected_components.connected_components(
-            a, b
-        ),
-        "co_equalizer": lambda a, b: data_algebra.connected_components.connected_components(
-            a, b
-        ),
-        "mapv": _map_v,
-        # additonal fns
-        # x is a pandas Series
-        "as_int64": lambda x: x.astype("int64").copy(),
-        "as_str": lambda x: x.astype("str").copy(),
-        "trimstr": lambda x, start, stop: x.str.slice(start=start, stop=stop),
-        "datetime_to_date": lambda x: x.dt.date.copy(),
-        "parse_datetime": lambda x, format: data_algebra.default_data_model.pd.to_datetime(
-            x, format=format
-        ),
-        "parse_date": lambda x, format: data_algebra.default_data_model.pd.to_datetime(
-            x, format=format
-        ).dt.date.copy(),
-        "format_datetime": lambda x, format: x.dt.strftime(date_format=format),
-        "format_date": lambda x, format: data_algebra.default_data_model.pd.to_datetime(
-            x
-        ).dt.strftime(date_format=format),
-        "dayofweek": lambda x: 1
-        + (
-            (
-                data_algebra.default_data_model.pd.to_datetime(x).dt.dayofweek.astype(
-                    "int64"
-                )
-                + 1
-            )
-            % 7
-        ),
-        "dayofyear": lambda x: data_algebra.default_data_model.pd.to_datetime(x)
-        .dt.dayofyear.astype("int64")
-        .copy(),
-        "weekofyear": _calc_week_of_Year,
-        "dayofmonth": lambda x: data_algebra.default_data_model.pd.to_datetime(x)
-        .dt.day.astype("int64")
-        .copy(),
-        "month": lambda x: data_algebra.default_data_model.pd.to_datetime(x)
-        .dt.month.astype("int64")
-        .copy(),
-        "quarter": lambda x: data_algebra.default_data_model.pd.to_datetime(x)
-        .dt.quarter.astype("int64")
-        .copy(),
-        "year": lambda x: data_algebra.default_data_model.pd.to_datetime(x)
-        .dt.year.astype("int64")
-        .copy(),
-        "timestamp_diff": lambda c1, c2: [
-            data_algebra.default_data_model.pd.Timedelta(c1[i] - c2[i]).total_seconds()
-            for i in range(len(c1))
-        ],
-        "date_diff": _calc_date_diff,
-        "base_Sunday": _calc_base_Sunday,
-    }
-    return impl_map
-
-
 # base class for Pandas-like API realization
 class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     """
@@ -308,7 +160,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             self, presentation_model_name=presentation_model_name
         )
         self.pd = pd
-        self.impl_map = populate_impl_map(data_model=self)
+        self.impl_map = self._populate_impl_map()
         self.transform_op_map = {"any_value": "first"}
         self.user_fun_map = dict()
         self._method_dispatch_table = {
@@ -327,6 +179,190 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             "TableDescription": self.table_step,
         }
 
+    # implementations
+    
+    def _calc_date_diff(self, x0, x1):
+        # x is a pandas Series or list of datetime.date compatible types
+        x0 = self.pd.Series(x0)
+        x1 = self.pd.Series(x1)
+        x0_dates = self.pd.to_datetime(x0).dt.date.copy()
+        x1_dates = self.pd.to_datetime(x1).dt.date.copy()
+        deltas = [(x0_dates[i] - x1_dates[i]).days for i in range(len(x0_dates))]
+        return deltas
+
+    def _calc_base_Sunday(self, x):
+        # x is a pandas Series or list of datetime.date compatible types
+        x = self.pd.Series(x)
+        x_dates = self.pd.to_datetime(x).dt.date.copy()
+        res = [xi - datetime.timedelta(days=(xi.weekday() + 1) % 7) for xi in x_dates]
+        return res
+
+    def _calc_week_of_Year(self, x):
+        # x is a pandas Series or list of datetime.date compatible types
+        # TODO: better impl
+        # Note was getting inconsistent results on vectorized methods
+        x = self.pd.Series(x)
+        x_dates = self.pd.to_datetime(x).dt.date.copy()
+        cur_dates = [datetime.date(dti.year, dti.month, dti.day) for dti in x_dates]
+        base_dates = [datetime.date(dti.year, 1, 1) for dti in x_dates]
+        base_dates = self._calc_base_Sunday(base_dates)
+        deltas = [(cur_dates[i] - base_dates[i]).days for i in range(len(cur_dates))]
+        res = [di // 7 for di in deltas]
+        res = numpy.maximum(res, 1)
+        return res
+
+    def _coalesce(self, a, b):
+        a_is_series = isinstance(a, self.pd.Series)
+        b_is_series = isinstance(b, self.pd.Series)
+        if (not a_is_series) and (not b_is_series):
+            raise ValueError("at least one argument must be a Pandas series")
+        if not a_is_series:
+            a = self.pd.Series(numpy.array([a] * len(b)))
+        if not b_is_series:
+            b = self.pd.Series(numpy.array([b] * len(a)))
+        res = a.combine_first(b)
+        return res
+
+    def _map_v(self, a, value_map, default_value=None):
+        """Map values to values."""
+        if len(value_map) > 0:
+            type_a = data_algebra.util.guess_carried_scalar_type(a)
+            type_k = {
+                data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.keys()
+            }
+            if not data_algebra.util.compatible_types(type_k):
+                raise TypeError(f"multiple types in dictionary keys: {type_k} in mapv()")
+            type_k = list(type_k)[0]
+            if not data_algebra.util.compatible_types([type_a, type_k]):
+                raise TypeError(f"can't map {type_a} from a dict of {type_k}'s in mapv()")
+            type_v = {
+                data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.values()
+            }
+            if not data_algebra.util.compatible_types(type_v):
+                raise TypeError(f"multiple types in dictionary values: {type_v} in mapv()")
+            type_v = list(type_v)[0]
+            if default_value is not None:
+                type_d = data_algebra.util.guess_carried_scalar_type(default_value)
+                if not data_algebra.util.compatible_types([type_d, type_v]):
+                    raise TypeError(f"default is {type_d} for {type_v} values in mapv()'s")
+        # https://pandas.pydata.org/docs/reference/api/pandas.Series.map.html
+        a = self.pd.Series(a)
+        a = a.map(value_map, na_action="ignore")
+        a[self.bad_column_positions(a)] = default_value
+        return numpy.array(a.values)
+
+    def _if_else_expr(self, *args):
+        """
+        if_else(cond, a, b) returns a for positions where cond is True, b when cond is False, None otherwise.
+        """
+        assert len(args) == 3
+        cond = args[0]
+        a = args[1]
+        b = args[2]
+        res = numpy.where(cond, a, b)
+        bad_posns = self.bad_column_positions(cond)
+        if numpy.any(bad_posns):
+            res[bad_posns] = None
+        return res
+
+    def _populate_impl_map(self) -> Dict[str, Callable]:
+        """
+        Map symbols to implementations.
+        """
+        impl_map = {
+            "==": _type_safe_equal,
+            "=": _type_safe_equal,
+            "!=": _type_safe_not_equal,
+            "<>": _type_safe_not_equal,
+            "<": numpy.less,  # already checks types
+            "<=": numpy.less_equal,  # already checks types
+            ">": numpy.greater,  # already checks types
+            ">=": numpy.greater_equal,  # already checks types
+            "+": _k_add,
+            "-": _negate_or_subtract,
+            "neg": numpy.negative,
+            "*": _k_mul,
+            "/": numpy.divide,
+            "//": numpy.floor_divide,
+            "%/%": numpy.divide,
+            "%": numpy.mod,
+            "**": numpy.power,
+            "and": _k_and,
+            "&": numpy.bitwise_and,
+            "or": _k_or,
+            "|": _k_or,
+            "xor": numpy.logical_xor,
+            "^": numpy.logical_xor,
+            "not": numpy.bitwise_or,
+            "where": _where_expr,
+            "if_else": lambda *args: self._if_else_expr(*args),
+            "is_nan": self.isnan,
+            "is_inf": self.isinf,
+            "is_null": self.isnull,
+            "is_bad": self.bad_column_positions,
+            "is_in": _type_safe_is_in,
+            "concat": lambda a, b: numpy.char.add(
+                numpy.asarray(a, dtype=str), numpy.asarray(b, dtype=str)
+            ),
+            "coalesce": lambda a, b: self._coalesce(a, b),  # assuming Pandas series
+            "connected_components": lambda a, b: data_algebra.connected_components.connected_components(
+                a, b
+            ),
+            "co_equalizer": lambda a, b: data_algebra.connected_components.connected_components(
+                a, b
+            ),
+            "mapv": lambda a, value_map, default_value=None: self._map_v(a, value_map, default_value),
+            # additonal fns
+            # x is a pandas Series
+            "as_int64": lambda x: x.astype("int64").copy(),
+            "as_str": lambda x: x.astype("str").copy(),
+            "trimstr": lambda x, start, stop: x.str.slice(start=start, stop=stop),
+            "datetime_to_date": lambda x: x.dt.date.copy(),
+            "parse_datetime": lambda x, format: self.pd.to_datetime(
+                x, format=format
+            ),
+            "parse_date": lambda x, format: self.pd.to_datetime(
+                x, format=format
+            ).dt.date.copy(),
+            "format_datetime": lambda x, format: x.dt.strftime(date_format=format),
+            "format_date": lambda x, format: self.pd.to_datetime(
+                x
+            ).dt.strftime(date_format=format),
+            "dayofweek": lambda x: 1
+            + (
+                (
+                    self.pd.to_datetime(x).dt.dayofweek.astype(
+                        "int64"
+                    )
+                    + 1
+                )
+                % 7
+            ),
+            "dayofyear": lambda x: self.pd.to_datetime(x)
+            .dt.dayofyear.astype("int64")
+            .copy(),
+            "weekofyear": lambda x: self._calc_week_of_Year(x),
+            "dayofmonth": lambda x: self.pd.to_datetime(x)
+            .dt.day.astype("int64")
+            .copy(),
+            "month": lambda x: self.pd.to_datetime(x)
+            .dt.month.astype("int64")
+            .copy(),
+            "quarter": lambda x: self.pd.to_datetime(x)
+            .dt.quarter.astype("int64")
+            .copy(),
+            "year": lambda x: self.pd.to_datetime(x)
+            .dt.year.astype("int64")
+            .copy(),
+            "timestamp_diff": lambda c1, c2: [
+                self.pd.Timedelta(c1[i] - c2[i]).total_seconds()
+                for i in range(len(c1))
+            ],
+            "date_diff": lambda x0, x1: self._calc_date_diff(x0, x1),
+            "base_Sunday": lambda x: self._calc_base_Sunday(x),
+        }
+        return impl_map
+
     # utils
 
     def data_frame(self, arg=None):
@@ -441,9 +477,10 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = db_handle.read_query("\n".join(op.sql))
         return res
 
-    def columns_to_frame_(self, cols, *, target_rows=0):
+    def columns_to_frame_(self, cols: Dict[str, Any], *, target_rows: Optional[int] = None):
         """
         Convert a dictionary of column names to series-like objects and scalars into a Pandas data frame.
+        Deal with special cases, such as some columns coming in as scalars (often from Panda aggregation).
 
         :param cols: dictionary mapping column names to columns
         :param target_rows: number of rows we are shooting for
@@ -451,46 +488,37 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         """
         # noinspection PyUnresolvedReferences
         assert isinstance(cols, dict)
+        assert isinstance(target_rows, (int, type(None)))
+        if target_rows is not None:
+            assert target_rows >= 0
         if len(cols) < 1:
-            return self.pd.DataFrame(cols)
-        for k, v in cols.items():
-            try:
-                target_rows = max(target_rows, len(v))
-            except TypeError:
-                target_rows = max(target_rows, 1)  # scalar
+            # all scalars, so nothing carrying index information
+            if target_rows is not None:
+                return self.pd.DataFrame({}, index=range(target_rows)).reset_index(drop=True, inplace=False)
+            else:
+                return self.pd.DataFrame({})
+        was_all_scalars = True
+        for v in cols.values():
+            ln = none_mark_scalar_or_length(v)
+            if ln is not None:
+                was_all_scalars = False
+                if target_rows is None:
+                    target_rows = ln
+                else:
+                    assert target_rows == ln
+        if was_all_scalars:
+            if target_rows is None:
+                target_rows = 1
+            # all scalars, so nothing carrying index information
+            promoted_cols = {k: promote_scalar_to_array(v, target_len=target_rows) for (k, v) in cols.items()}
+            return self.pd.DataFrame(promoted_cols, index=range(target_rows)).reset_index(drop=True, inplace=False)
+        assert target_rows is not None
         if target_rows < 1:
-            # noinspection PyBroadException
-            try:
-                res = self.pd.DataFrame(cols)
-                if res.shape[0] > 0:
-                    res = res.loc[[False] * res.shape[0], :].reset_index(
-                        drop=True, inplace=False
-                    )
-            except Exception:
-                res = self.pd.DataFrame({k: [] for k in cols.keys()})
-            return res
-
+            # no rows, so presuming no index information (shouldn't have come from an aggregation)
+            return self.pd.DataFrame({k: [] for k in cols.keys()})
         # agg can return scalars, which then can't be made into a self.pd.DataFrame
-        def promote_scalar(vi, *, target_len):
-            """
-            Convert a scalar into a vector.
-            """
-            # noinspection PyBroadException
-            try:
-                len_v = len(vi)
-                if len_v != target_len:
-                    if len_v == 0:
-                        return [None] * target_len
-                    elif len_v == 1:
-                        return [vi[0]] * target_len
-                    else:
-                        raise ValueError("incompatible column lengths")
-            except Exception:
-                return [vi] * target_len  # scalar
-            return vi
-
-        cols = {k: promote_scalar(v, target_len=target_rows) for (k, v) in cols.items()}
-        return self.pd.DataFrame(cols)
+        promoted_cols = {k: promote_scalar_to_array(v, target_len=target_rows) for (k, v) in cols.items()}
+        return self.pd.DataFrame(promoted_cols)
 
     def add_data_frame_columns_to_data_frame_(self, res, transient_new_frame):
         """
@@ -547,6 +575,15 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         """
         if op.node_name != "ExtendNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ExtendNode")
+        res = self._eval_value_source(op.sources[0], data_map=data_map, narrow=narrow)
+        if res.shape[0] <= 0:
+            # special case out no-row frame
+            incoming_col_set = set(res.columns)
+            v_dict = {k: [] for k in res.columns}
+            for k in op.ops.keys():
+                if k not in incoming_col_set:
+                    v_dict[k] = []
+            return self.pd.DataFrame(v_dict)
         window_situation = (
             op.windowed_situation
             or (len(op.partition_by) > 0)
@@ -554,7 +591,6 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         )
         if window_situation:
             op.check_extend_window_fns_()
-        res = self._eval_value_source(op.sources[0], data_map=data_map, narrow=narrow)
         if not window_situation:
             with warnings.catch_warnings():
                 warnings.simplefilter(
@@ -574,7 +610,6 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
                     col_list = col_list + [c]
                     col_set.add(c)
             order_cols = [c for c in col_list]  # must be partition by followed by order
-
             for (k, opk) in op.ops.items():
                 # assumes all args are column names or values, enforce this earlier
                 if len(opk.args) > 0:
@@ -756,10 +791,15 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         # agg can return scalars, which then can't be made into a self.pd.DataFrame
         res = self.columns_to_frame_(cols)
         res = res.reset_index(
-            drop=len(op.group_by) < 1
+            drop=(len(op.group_by) < 1) or (res.shape[0] <= 0)
         )  # grouping variables in the index
         missing_group_cols = set(op.group_by) - set(res.columns)
-        assert len(missing_group_cols) <= 0
+        if res.shape[0] > 0:
+            if len(missing_group_cols) != 0:
+                raise ValueError("Missing column groups")
+        else:
+            for g in missing_group_cols:
+                res[g] = []
         if "_data_table_temp_col" in res.columns:
             res = res.drop("_data_table_temp_col", axis=1, inplace=False)
         # double check shape is what we expect
@@ -878,6 +918,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             )
         left = self._eval_value_source(op.sources[0], data_map=data_map, narrow=narrow)
         right = self._eval_value_source(op.sources[1], data_map=data_map, narrow=narrow)
+        if (left.shape[0] == 0) and (right.shape[0] == 0):
+            # pandas seems to not like this case
+            return self.pd.DataFrame({k: [] for k in op.columns_produced()})
         common_cols = set([c for c in left.columns]).intersection(
             [c for c in right.columns]
         )
