@@ -2,9 +2,6 @@
 Adapter for Pandas API.
 """
 
-
-import pandas as pd
-
 from data_algebra.pandas_base import PandasModelBase
 
 
@@ -14,4 +11,9 @@ class PandasModel(PandasModelBase):
     """
 
     def __init__(self):
+        import pandas as pd
         PandasModelBase.__init__(self, pd=pd, presentation_model_name="pd")
+
+
+# set up what pandas supplier we are using
+default_data_model = PandasModel()
